//
//  ContextMenu.cpp
//  gtk_channels
//
//  Created by Arnaud Blanchard on 13/10/2019.
//

#include "ChannelWidget.hpp"
#include "blc_image.h"

using namespace Gtk;
using namespace std;

ContextMenu::ContextMenu(string program_dir):program_dir(move(program_dir)){}

void ContextMenu::add_item(string name, bool shortcut, function <void()> action){
    MenuItem *item=new MenuItem(name, shortcut);
    append(*item);
    item->signal_activate().connect(action);
}

void ContextMenu::add_separator(){
    SeparatorMenuItem *separator_item = new SeparatorMenuItem();
    append(*separator_item);
}

void ContextMenu::add_program(char const *menu_name, std::string program_name, initializer_list<const char*> const optional_arguments, function <void(Process *process)> after_launch){
    auto item=new MenuItem(menu_name);
    auto args_vec=make_shared<vector<const char*>>(optional_arguments); //This will be destroyed by process
    append(*item);
    
    item->signal_activate().connect([this, args_vec, program_name, after_launch]{
        args_vec->reserve(args_vec->size()+selected_channels.size());
        for(auto const &channel:selected_channels){
            args_vec->push_back(strdup(channel->name));
        }
        auto process = new Process(program_dir, program_name, args_vec);
        process->run(after_launch);
    });
}
