//
//  Process.cpp
//  e_bash
//
//  Created by Arnaud Blanchard on 30/08/2017.
//
//
#include "Process.h"
#include "blc_core.h"
#include <errno.h>
#include <unistd.h> //execve
#include <thread>
#include <vte/vte.h>
#include <sys/pipe.h>
#include <gtkmm.h>
#include <signal.h>


using namespace std;
using namespace Gtk;

static void TerminalSpawnAsyncCallback (VteTerminal *terminal, GPid pid, GError *error, gpointer user_data){
    Process *process=(Process*)user_data;
    if (error){
        fprintf(stderr, "Process error: '%s'\n", error->message);
        for (auto const &arg : process->arg_vector){
            fprintf(stderr, "%s ", arg);
        }
        fprintf(stderr, "\n");
    }
    else {
        process->pid=pid;
        fprintf(stderr, "Success pid %d\n", pid);
        for (auto const &arg : process->arg_vector){
                   fprintf(stderr, "%s ", arg);
               }
               fprintf(stderr, "\n");
        process->after_launch(process);
    }
};

static void after_exectution(VteTerminal *vteterminal, gint status, gpointer user_data){
    auto process=(Process*)user_data;
    process->on_quit_callback(status);
}


void Process::on_quit(function<void(int exit_status)> on_quit){
    on_quit_callback=on_quit;
    g_signal_connect(G_OBJECT(terminal), "child-exited", G_CALLBACK(after_exectution) , this);
}


Process::Process(string const directory, string const program, shared_ptr<vector<const char*>> const optional_arguments):directory(move(directory)),program_name(move(program)){
    command_path=directory+"/"+program;
    arg_vector.emplace_back(command_path.data());
    
    for( auto &arg : *optional_arguments){
        arg_vector.emplace_back(arg);
    }
};

Process::~Process(){
    for (char const *&arg: arg_vector ){
        free(&arg);
    }
};

void Process::run(function <void(Process *process)> after_launch){
    this->after_launch=after_launch;
    terminal=VTE_TERMINAL(vte_terminal_new());
    arg_vector.emplace_back(nullptr); //note the end of the list
    vte_terminal_spawn_async(terminal, VTE_PTY_DEFAULT , nullptr, (char**)arg_vector.data(), nullptr, G_SPAWN_DEFAULT, nullptr, nullptr, nullptr, 1000, nullptr, TerminalSpawnAsyncCallback, (void*)this);
}

void Process::display_terminal(){
    Widget *mm_terminal=Glib::wrap(GTK_WIDGET(terminal)); //This is because vte is not in C++/gtkmm
    
    if (terminal_window==nullptr){
        terminal_window=new Window();
        terminal_window->add(*mm_terminal);
        terminal_window->signal_delete_event().connect([this](GdkEventAny*){
            terminal_window->hide();
            return true;
        });
    }
    terminal_window->show_all();
}

void Process::signal(int signal) const{
    kill(pid, signal);
}
